import numpy
import sys
import os
import math
import yaml
import types
import matplotlib.pyplot as plt


# select the visual style for images
plt.style.use('classic')
#print(plt.style.available)


# configuration
impact_type = "LinearForce"    # options are "ControlPoint", "LinearForce"
control_spring_stiffness = 100000000
spring_type = "PolynomialRestShape"    # options are "Polynomial" and "PolynomialRestShape"
initial_spring_length = 0.02 # 0.021213203
polynomial_stiffness = [30.0, -500.0, 3000.0] # [8.0, 15.0, 20.0]


###
### load location data from files
###
def loadDataFromMonitorFile(File):
    fo = open(File, 'r')
    # throw out first line
    fo.readline()
    fo.readline()
    line = fo.readline()

    tokens = line.split()
    tokens = [float(elem) for elem in tokens]
    data_array = numpy.empty((0, len(tokens) - 1))
    while line:
        tokens = line.split()
        tokens = [float(elem) for elem in tokens]
        data_array = numpy.append(data_array, [tokens[1:]], axis=0)
        line = fo.readline()

    return numpy.matrix(data_array)


def loadDataFromFile(File):
    fo = open(File, 'r')
    line = fo.readline()

    tokens = line.split()
    tokens = [float(elem) for elem in tokens]
    data_array = numpy.empty((0, len(tokens) - 1))
    while line:
        tokens = line.split()
        tokens = [float(elem) for elem in tokens]
        data_array = numpy.append(data_array, [tokens[1:]], axis=0)
        line = fo.readline()

    return numpy.matrix(data_array)


# create color map object
cmap = plt.cm.get_cmap('hsv', 10)


if impact_type == 'ControlPoint':

    controlPointFile = 'data/brickCPimpact_x.txt'
    observedPointFile = 'data/pointForceField_x.txt'
    observedPointForce = 'data/pointForceField_f.txt'

    controlPointLocation = loadDataFromMonitorFile(controlPointFile)
    paramsAmount = numpy.size(controlPointLocation[1,:])
    timeLength = numpy.size(controlPointLocation[:,1])
    x_data = xrange(0, timeLength)

    pointObservedPointLocation = loadDataFromMonitorFile(observedPointFile)
    observedPointForce = loadDataFromMonitorFile(observedPointForce)


    ### plot locations
    fig1 = plt.figure(1)
    spl1 = fig1.add_subplot(111)

    for index in range(0, paramsAmount):
        spl1.plot(x_data, controlPointLocation[:, index], color=cmap(index), linestyle='solid', linewidth=4, label='control point location')
        spl1.plot(x_data, pointObservedPointLocation[:, index], color=cmap(index + 3), linestyle='solid', linewidth=4, label='observed point location')
    # compute difference
    diff = abs(pointObservedPointLocation[0:timeLength, :] - controlPointLocation[0:timeLength,:])
    diffNorm = numpy.zeros(timeLength)
    for index in range (0, timeLength):
        diffNorm[index] = math.sqrt(diff.item((index, 0))*diff.item((index, 0)) + diff.item((index, 1))*diff.item((index, 1)) + diff.item((index, 2))*diff.item((index, 2)))

    spl1.plot(x_data, diffNorm, color=cmap(7), linestyle='solid', linewidth=5, label='difference')
    spl1.set_xlabel('iterations', fontsize=50)
    spl1.set_ylabel('distance, m', fontsize=50)
    spl1.tick_params(axis = 'both', which = 'major', labelsize=40)
    spl1.grid(color='k', linestyle=':', linewidth=1)
    spl1.set_title('Point locations: ')
    legendForSpl1 = spl1.legend(loc='upper center', shadow=True, fontsize='x-large')
    legendForSpl1.get_frame().set_facecolor('#FFFFFF')


    ### plot forces
    fig2 = plt.figure(2)
    spl2 = fig2.add_subplot(111)

    linearForceStiffness = control_spring_stiffness
    linearForceDiff = linearForceStiffness * diff
    for index in range(0, paramsAmount):
        spl2.plot(x_data, observedPointForce[:, index], color=cmap(1), linestyle='solid', linewidth=4, label='observed force values')
        spl2.plot(x_data, linearForceDiff[:, index], color=cmap(2), linestyle='solid', linewidth=8, label='linear force values')
    # compute difference
    forceDiff = abs(observedPointForce[0:timeLength, :] - linearForceDiff[0:timeLength,:])
    pointForceNorm = numpy.zeros(timeLength)
    for index in range (0, timeLength):
        pointForceNorm[index] = math.sqrt(forceDiff.item((index, 0))*forceDiff.item((index, 0)) + forceDiff.item((index, 1))*forceDiff.item((index, 1)) + forceDiff.item((index, 2))*forceDiff.item((index, 2)))
    spl2.plot(x_data, pointForceNorm, color=cmap(8), linestyle='solid', linewidth=4, label='force difference')

    spl2.set_xlabel('iterations', fontsize=50)
    spl2.set_ylabel('forces, N', fontsize=50)
    spl2.tick_params(axis = 'both', which = 'major', labelsize=40)
    spl2.grid(color='k', linestyle=':', linewidth=1)
    spl2.set_title('Forces values:')
    legendForSpl2 = spl2.legend(loc='upper center', shadow=True, fontsize='x-large')
    legendForSpl2.get_frame().set_facecolor('#FFFFFF')


if impact_type == 'LinearForce':

    appliedForceFile = 'data/forceData.txt'
    observedPointFile = 'data/pointForceField_x.txt'
    observedPointForce = 'data/pointForceField_f.txt'

    appliedForce = loadDataFromMonitorFile(appliedForceFile)
    forceObservedPointLocation = loadDataFromMonitorFile(observedPointFile)
    observedPointForce = loadDataFromMonitorFile(observedPointForce)
    shift = int(numpy.size(appliedForce[:,1]))
    appliedForce = appliedForce[1:shift, :]
    #for index in range(0, shift-1):
    #    appliedForce[index] = appliedForce[index] * 1000
    forceObservedPointLocation = forceObservedPointLocation[1:shift, :]
    observedPointForce = observedPointForce[1:shift, :]

    paramsAmount = numpy.size(forceObservedPointLocation[1,:])
    timeLength = numpy.size(forceObservedPointLocation[:,1])
    x_data = xrange(0, timeLength)


    ### plot locations
    fig3 = plt.figure(3)
    spl3 = fig3.add_subplot(111)

    for index in range(0, paramsAmount):
        spl3.plot(x_data, forceObservedPointLocation[:, index], color=cmap(1), linestyle='solid', linewidth=4, label='observed point location')
    # compute difference
    spl3.set_xlabel('iterations', fontsize=50)
    spl3.set_ylabel('distance, m', fontsize=50)
    spl3.tick_params(axis = 'both', which = 'major', labelsize=40)
    spl3.grid(color='k', linestyle=':', linewidth=1)
    spl3.set_title('Linear force locations: ')
    legendForSpl3 = spl3.legend(loc='upper center', shadow=True, fontsize='x-large')
    legendForSpl3.get_frame().set_facecolor('#FFFFFF')


    ### plot forces
    fig4 = plt.figure(4)
    spl4 = fig4.add_subplot(111)

    # draw forces
    for index in range(0, paramsAmount):
        spl4.plot(x_data, observedPointForce[:, index], color=cmap(1), linestyle='solid', linewidth=4, label='observed force values')
        spl4.plot(x_data, appliedForce[:, index], color=cmap(2), linestyle='solid', linewidth=4, label='linear force values')
    # compute difference
    forceDiff = abs(observedPointForce[0:timeLength, :] - appliedForce[0:timeLength,:])
    forceForceNorm = numpy.zeros(timeLength)
    for index in range (0, timeLength):
        forceForceNorm[index] = math.sqrt(forceDiff.item((index, 0))*forceDiff.item((index, 0)) + forceDiff.item((index, 1))*forceDiff.item((index, 1)) + forceDiff.item((index, 2))*forceDiff.item((index, 2)))
    spl4.plot(x_data, forceForceNorm, color=cmap(8), linestyle='solid', linewidth=5, label='force difference')

    spl4.set_xlabel('iterations', fontsize=50)
    spl4.set_ylabel('forces, N', fontsize=50)
    spl4.tick_params(axis = 'both', which = 'major', labelsize=40)
    spl4.grid(color='k', linestyle=':', linewidth=1)
    spl4.set_title('Forces values:')
    legendForSpl4 = spl4.legend(loc='upper center', shadow=True, fontsize='x-large')
    legendForSpl4.get_frame().set_facecolor('#FFFFFF')


### plot stress-strain relation (general for all scenes) ###
fig5 = plt.figure(5)
spl5 = fig5.add_subplot(111)

#timeLength = min(numpy.size(controlPointLocation[:,1]), numpy.size(forceObservedPointLocation[:,1]))

if impact_type == 'ControlPoint':
    pointObservedLocationDistance = numpy.zeros(timeLength)
    for index in range (0, timeLength):
        pointObservedLocationDistance[index] = math.sqrt(pointObservedPointLocation.item((index, 0))*pointObservedPointLocation.item((index, 0)) + pointObservedPointLocation.item((index, 1))*pointObservedPointLocation.item((index, 1)) + pointObservedPointLocation.item((index, 2))*pointObservedPointLocation.item((index, 2)))
        if spring_type == "PolynomialRestShape":
            pointObservedLocationDistance[index] = pointObservedLocationDistance[index] / initial_spring_length
        elif spring_type == "Polynomial":
            pointObservedLocationDistance[index] = (pointObservedLocationDistance[index] - initial_spring_length) / initial_spring_length
    spl5.plot(pointObservedLocationDistance, pointForceNorm[0:timeLength], color='gold', linestyle='solid', linewidth=12, label='force stretch relation - imposed displacement', zorder = 1)

if impact_type == 'LinearForce':
    forceObservedLocationDistance = numpy.zeros(timeLength)
    for index in range (0, timeLength):
        forceObservedLocationDistance[index] = math.sqrt(forceObservedPointLocation.item((index, 0))*forceObservedPointLocation.item((index, 0)) + forceObservedPointLocation.item((index, 1))*forceObservedPointLocation.item((index, 1)) + forceObservedPointLocation.item((index, 2))*forceObservedPointLocation.item((index, 2)))
        if spring_type == "PolynomialRestShape":
            forceObservedLocationDistance[index] = forceObservedLocationDistance[index] / initial_spring_length
        elif spring_type == "Polynomial":
            forceObservedLocationDistance[index] = (forceObservedLocationDistance[index] - initial_spring_length) / initial_spring_length
    spl5.plot(forceObservedLocationDistance, forceForceNorm[0:timeLength], color='g', linestyle='solid', linewidth=6, label='force stretch relation - increasing force', zorder = 2)

# draw analitycal function
y_values = numpy.zeros(timeLength)
for index in range (0, timeLength):
    polynomial_strain = 1.0
    for internalIndex in range (0, len(polynomial_stiffness)):
        if impact_type == 'ControlPoint':
            polynomial_strain = polynomial_strain * pointObservedLocationDistance[index]
        elif impact_type == 'LinearForce':
            polynomial_strain = polynomial_strain * forceObservedLocationDistance[index]
        y_values[index] = y_values[index] + polynomial_stiffness[internalIndex] * polynomial_strain
if impact_type == 'ControlPoint':
    spl5.plot(pointObservedLocationDistance, y_values, color=cmap(5), linestyle='solid', linewidth=3, label='polynomial result', zorder = 3)
elif impact_type == 'LinearForce':
    spl5.plot(forceObservedLocationDistance, y_values, color=cmap(5), linestyle='solid', linewidth=3, label='polynomial result', zorder = 3)

### compute maximum difference
controlPointMax = 0.0
linearForceMax = 0.0
point_y_values = numpy.zeros(timeLength)
force_y_values = numpy.zeros(timeLength)
for index in range (10, timeLength):
    if impact_type == 'ControlPoint':
        polynomial_strain = 1.0
        for internalIndex in range (0, len(polynomial_stiffness)):
            polynomial_strain = polynomial_strain * pointObservedLocationDistance[index]
            point_y_values[index] = point_y_values[index] + polynomial_stiffness[internalIndex] * polynomial_strain
        controlPointMax = controlPointMax + abs(point_y_values[index] - pointForceNorm[index]) / pointForceNorm[index] * 100
        controlPointMax = controlPointMax / (timeLength - 10)

    elif impact_type == 'LinearForce':
        polynomial_strain = 1.0
        for internalIndex in range (0, len(polynomial_stiffness)):
            polynomial_strain = polynomial_strain * forceObservedLocationDistance[index]
            force_y_values[index] = force_y_values[index] + polynomial_stiffness[internalIndex] * polynomial_strain
        linearForceMax = linearForceMax + abs(force_y_values[index] - forceForceNorm[index]) / forceForceNorm[index] * 100
        linearForceMax = linearForceMax / (timeLength - 10)
if impact_type == 'ControlPoint':
    print 'Maximal difference in estimations: controlPoint: ' + str(controlPointMax)
elif impact_type == 'LinearForce':
    print 'Maximal difference in estimations: linearForce: ' + str(linearForceMax)


# draw axes and legend
spl5.set_xlabel('Strain', fontsize=50)
spl5.set_ylabel('Force norm, Pa', fontsize=50)
#spl5.set_xlim(0.0, 10.0)
#spl5.set_ylim(0.0, 100000000)
spl5.tick_params(axis = 'both', which = 'major', labelsize=40)
spl5.grid(color='k', linestyle=':', linewidth=1)
spl5.set_title('Experiment: a0 = 0.0, a1 = '+str(polynomial_stiffness[0])+', a2 = '+str(polynomial_stiffness[1])+', a3 = '+str(polynomial_stiffness[2]),fontsize=40)
legendForSpl5 = spl5.legend(loc='upper left', shadow=True, fontsize=40)
legendForSpl5.get_frame().set_facecolor('#FFFFFF')


plt.show()

